
var timelineInfo = {
	scaleList : {
		0 : {scale : 'month', month : 1, year : 0, unitHeight: 400}	//'month'
	   ,1 : {scale : 'month', month : 1, year : 0, unitHeight: 200}	//'month'
	   ,2 : {scale : 'month', month : 1, year : 0, unitHeight: 100}	//'month'
	   ,3 : {scale : 'year', month : 12, year : 1, unitHeight: 200}	//'year'
	   ,4 : {scale : 'year', month : 12, year : 1, unitHeight: 100}	//'year'
	   ,5 : {scale : '2year', month : 24, year : 2, unitHeight: 100}	//'2year'
	   ,6 : {scale : '10year', month : 120, year : 10, unitHeight: 200}	//'10year'
	}
   ,scaleNoMin : 0	//スケールNoの最小値
   ,scaleNoMax : 6	//スケールNoの最大値
   ,scaleNo : 3	//現在のスケールのindex
   ,scale : 'month'	//現在のスケール(year)
   ,unitHeight : 200	//1メモリの高さ(px)
   ,unitDays : 0	//現スケールでの目盛り間の日数
   ,pixelPerDay : 0	//現スケールでの1日あたりのpx

   ,startDate: null	//最古の日付
   ,endDate: null	//最新日付
   ,scaleStartDate : null	//最古のメモリ日
   ,scaleEndDate : null		//最新のメモリ日
   ,marginTop: 100
   ,isNoDate: true	//日付なし
   ,defaultHeight: 1000	//日付なしの場合の既定の高さ
};


/**
 * タイムライン表示
 */
function timelineSet()
{
	//開始日、終了日を取得
	timelineInfo.endDate = timelineToDate($('#timeline_end_date').val(), '-');
	timelineInfo.startDate = timelineToDate($('#timeline_start_date').val(), '-');

	if (isNaN(timelineInfo.endDate.valueOf()) || isNaN(timelineInfo.startDate.valueOf())){
		timelineInfo.isNoDate = true;
	} else {
		timelineInfo.isNoDate = false;
	}

	//既定スケールを決定
	timelineDefaultScale();
	//スケールによる画面の設定
	timelineSetScale();


}


/**
 * 現在のデータ、スケールから、必要メモリ数を求め、高さを決める
 */
function timelineSetScale()
{
	//目盛りの開始、終了日付を求める
	if (timelineInfo.isNoDate) {
		timelineInfo.scaleStartDate = null;
		timelineInfo.scaleEndDate = null;
	} else {
		timelineInfo.scaleStartDate = timelineGetScaleDate(timelineInfo.startDate, 'prev');
		timelineInfo.scaleEndDate = timelineGetScaleDate(timelineInfo.endDate, 'next');
	}

	//目盛りを振り、領域の高さを決定する
	timelineSetScaleDate();

	//イベントの位置を決定する
	timelineSetEventYpos();
}

/**
 * イベントの位置を決定する
 */
function timelineSetEventYpos()
{
	var unknownPos = timelineInfo.marginTop / 2;

	$('.b1njTimelineEvent').each(function(idx, item) {
		var elem = $(item);

		//イベント日付を取得
		var dateStr = elem.find('input.enforcement_date').val();
		var ypos = 0;
		
		if (dateStr == '') {
			//日付未設定なら固定位置
			ypos = unknownPos;
			unknownPos += 5;	//次に備えて位置をずらす
		} else {
			ypos = timelinePositonY(dateStr);
			if (ypos === null) {
				ypos = unknownPos;
				unknownPos += 5;	//次に備えて位置をずらす
			}
		}
		
		elem.css('top', '' + ypos + 'px');
	});
}


/**
 * 現在のスケールから日付のy座標(top:)を決定する
 */
function timelinePositonY(dateStr)
{
	var date = timelineToDate(dateStr, '-');
	var top = '0';
	var eventTop = 0; 

	//一致する目盛り日付があればその座標
	var scaleDateElem = $('#timeline_scale_' + timelineDateToStr2(date));
	if (scaleDateElem.length > 0) {
		top = scaleDateElem.css('top').replace('px', '');
		eventTop = Math.round(parseInt(top, 10));
		return eventTop;
	}


	//ひとつ上の目盛り日付を取得
	var scaleDate = timelineGetScaleDate(date, 'next');
	scaleDateElem = $('#timeline_scale_' + timelineDateToStr2(scaleDate));
	if (scaleDateElem.length <= 0) {
		//目盛り日付が取得できない
		return null;
	}

	//目盛り日付からの日数を取得
	var days = Math.abs(timelineDayCount(date, scaleDate));

	//目盛り日付のtopを取得
	top = scaleDateElem.css('top').replace('px', '');

	//日数から座標を算出
	eventTop = Math.round(parseInt(top, 10) + (days * timelineInfo.pixelPerDay));

	return eventTop;
}

/**
 * 目盛りを設定する
 */
function timelineSetScaleDate()
{
	var dateHtml = '';
	var ypos;
	timelineInfo.unitDays = 0;
	timelineInfo.pixelPerDay = 0;

	if (timelineInfo.isNoDate) {
		ypos = timelineInfo.defaultHeight;
	} else {
		var date = timelineInfo.scaleEndDate;
		var prevDate = date;
		var loopEndTime = timelineInfo.scaleStartDate.getTime();
		var limit = 9999;	//無限ループ対策
		ypos = timelineInfo.marginTop;
		while (prevDate.getTime() >= loopEndTime && limit > 0) {
			limit--;
			date = prevDate;
			dateHtml += ''
				+ '<li style="top: ' + ypos + 'px" id="timeline_scale_' + timelineDateToStr2(date) + '">'
				+ '<div>' + timelineGetScaleLabel(date) + '</div>'
				+ '<input type="hidden" class="timeline_scale_date_value" value="' + timelineDateToStr(date) + '">'
				+ '</li>'
				+ "\n";
			ypos += timelineInfo.unitHeight;
	
			var prevDate = timelineGetScaleDate(date, 'prev');
			
			//目盛り間の日数を保存しておく
			timelineInfo.unitDays = Math.max(timelineDayCount(prevDate, date), timelineInfo.unitDays);
		}
		//1日あたりのpxを保存しておく
		timelineInfo.pixelPerDay = timelineInfo.unitHeight / timelineInfo.unitDays;
	}

	$('.b1njTimeline .b1njTimelineDates').html(dateHtml);

	//高さを設定する
	$('.b1njTimeline').height(ypos - (timelineInfo.unitHeight / 2));
}

/**
 * 現在のスケールから日付目盛りのラベルを決定する
 */
function timelineGetScaleLabel(scaleDate)
{
	var wareki = Intl.DateTimeFormat('ja-JP-u-ca-japanese', {year: 'numeric', month: 'narrow', day: 'numeric', era: 'long'}).format(scaleDate);
	wareki = wareki.replace('令和', 'R');
	wareki = wareki.replace('平成', 'H');
	wareki = wareki.replace('昭和', 'S');
	wareki = wareki.replace('大正', 'T');
	wareki = wareki.replace('明治', 'M');

	var label = '';
	switch(timelineInfo.scaleList[timelineInfo.scaleNo].scale) {
		case 'month':
			//年月
			// label = wareki.replace(/月.*\d+.*日/, '月');
			// label = label.replace('年', '年<br>');
			label = wareki.replace(/月.*\d+.*日/, '');
			label = label.replace('年', '.');
			break;
		default:
			//月以外は年
			//label = wareki.replace(/年.*\d+.*月.*\d+.*日/, '年');
			label = wareki.replace(/年.*\d+.*月.*\d+.*日/, '');
			break;
	}

	label = label.replace('元.', '1.');
	return label;
}


/**
 * 開始日、終了日からタイムラインの既定スケールを決める
 */
function timelineDefaultScale()
{
	var scaleNo = 3;
	if (!timelineInfo.isNoDate) {
		//日付がある場合のみ
		var yearDiff = timelineInfo.endDate.getFullYear() - timelineInfo.startDate.getFullYear();
		if (yearDiff <= 3) {
			//開始日～終了日が3年以内なら月
			scaleNo = 2;
		} else if (yearDiff <= 10) {
			//10年以内なら年
			scaleNo = 4;
		} else if (yearDiff <= 30) {
			//30年以内なら2年
			scaleNo = 5;
		} else {
			//それ以上なら10年
			scaleNo = 6;
		}	
	}
	timelineSetCurrentScale(scaleNo);
}

/**
 * カレントスケールを設定する
 */
function timelineSetCurrentScale(scaleNo)
{
	timelineInfo.scaleNo = scaleNo;
	timelineInfo.scale = timelineInfo.scaleList[timelineInfo.scaleNo].scale;
	timelineInfo.unitHeight = timelineInfo.scaleList[timelineInfo.scaleNo].unitHeight;
}


/**
 * 指定日付の、現在のスケールの次日付、前日付を求める
 */
function timelineGetScaleDate(date, mode)
{
	//日付を年月日に分解
	var y = date.getFullYear();
	var m = date.getMonth();
	var d = date.getDate();

	if (mode == 'next') {
		//次日付
		switch(timelineInfo.scaleList[timelineInfo.scaleNo].scale){
			case 'month':
				//翌月の1日
				m++;
				d = 1;
				if (m >= 12) {
					m = 0;
					y++;
				}
				break;
			case 'year':
				//翌年の1月1日
				y++;
				m = 0;
				d = 1;
				break;
			case '2year':
				//翌2の倍数年の1月1日
				y += (timelineInfo.scaleList[timelineInfo.scaleNo].year - (y % timelineInfo.scaleList[timelineInfo.scaleNo].year));
				m = 0;
				d = 1;
				break;
			case '10year':
				//翌10の倍数年の1月1日
				y += (timelineInfo.scaleList[timelineInfo.scaleNo].year - (y % timelineInfo.scaleList[timelineInfo.scaleNo].year));
				m = 0;
				d = 1;
				break;
		}

		return new Date(y, m, d);
	} else {
		//前日付
		switch(timelineInfo.scaleList[timelineInfo.scaleNo].scale){
			case 'month':
				//前月の1日
				m--;
				if (m < 0) {
					m = 11;
					y--;
				}
				break;
			case 'year':
				//前年の1月1日
				y--;
				m = 0;
				d = 1;
				break;
			case '2year':
				//前2の倍数年の1月1日
				y -= (timelineInfo.scaleList[timelineInfo.scaleNo].year - (y % 2));
				m = 0;
				d = 1;
				break;
			case '10year':
				//前10の倍数年の1月1日
				y -= (timelineInfo.scaleList[timelineInfo.scaleNo].year - (y % 2));
				m = 0;
				d = 1;
				break;
		}

		return new Date(y, m, d);
	}
}

/**
 * 日付間の日数を返す(yyyy-mm-dd) 
 */
function timelineDayCountStr(fromDateStr, toDateStr)
{
	var fromDate = timelineToDate(fromDateStr, '-');
	var toDate = timelineToDate(toDateStr, '-');
	return timelineDayCount(formDate, toDate);
}

/**
 * 日付間の日数を返す(Date)
 */
function timelineDayCount(formDate, toDate)
{
	var days = (toDate.getTime() - formDate.getTime()) / (1000 * 60 * 60 * 24);
	return days;
}

/**
 * Date→yyyymmdd
 */
function timelineDateToStr2(date)
{
	var str = '' 
		+ date.getFullYear() 
		+ ('0' + (date.getMonth() + 1)).slice(-2) 
		+ ('0' + date.getDate()).slice(-2) ;
	return str;
}

/**
 * Date→yyyy-mm-dd
 */
function timelineDateToStr(date)
{
	var str = '' 
		+ date.getFullYear() 
		+ '-' + ('0' + (date.getMonth() + 1)).slice(-2) 
		+ '-' + ('0' + date.getDate()).slice(-2) ;
	return str;
}

/**
 * yyyy-mm-dd文字列→Date
 */
function timelineToDate (str, delim) {
	var arr = str.split(delim)
	return new Date(arr[0], arr[1] - 1, arr[2]);
}

/**
 * 時系列表示#outlineGraphの表示状態に合わせて拡大・縮小ボタン表示制御
 */
function timelineShowScaleControl()
{
	if ($(window).width() <= 1024) {
		//if ($('#fTocOutlineBtn').hasClass('active')) {
			if ($('#outlineGraph').is(':visible')) {
				$('.timelineScaleControl').addClass('active');
			} else {
				$('.timelineScaleControl').removeClass('active');
			}
		// } else {
		// 	$('.timelineScaleControl').removeClass('active');
		// }
	} else {
		//時系列表示なら表示
		if ($('#outlineGraph').is(':visible')) {
			$('.timelineScaleControl').addClass('active');
		} else {
			$('.timelineScaleControl').removeClass('active');
		}
	}
}

$(function(){
	/**
	 * IEのスクロール異常対策
	 */
	$('.timelineScaleControl button').on('focus', function(e){
		$('#timelineDummyBtn').focus();
	});

	/**
	 * タイムライン拡大、縮小
	 */
	$('.timelineScaleControl button.timelineScaleUpBtn').on('click', function(e){
		if (timelineInfo.scaleNo > timelineInfo.scaleNoMin) {
			//拡大の余地あり
			timelineSetCurrentScale(timelineInfo.scaleNo - 1);
			timelineSetScale();
			$('#timelineDummyBtn').focus();
		}
	});
	$('.timelineScaleControl button.timelineScaleDownBtn').on('click', function(e){
		if (timelineInfo.scaleNo < timelineInfo.scaleNoMax) {
			//縮小の余地あり
			timelineSetCurrentScale(timelineInfo.scaleNo + 1);
			timelineSetScale();
			$('#timelineDummyBtn').focus();
		}
	});

	/**
	 * 開く・閉じる
	 */
	$('.timeline_description').on('click', function(e){
		var elem = $(this);
		if (elem.hasClass('timelineContent_opened')) {
			//閉じる
			elem.removeClass('timelineContent_opened');
		} else {
			//開く
			elem.addClass('timelineContent_opened');
		}

		e.stopPropagation();
	});
	$('.timeline_subContnt').on('click', function(e){
		e.stopPropagation();
	});

	/**
	 * クリックで条文表示
	 */
	$('.timeline_topContnt, .timeline_subContent').on('click', function(e){
		var elem = $(this);
		var linkUniqueId = elem.find('input.link_unique_id').val();

		if (linkUniqueId != '') {
			var encodeUniqueId = linkUniqueId;
			if (userIE == 1) {
				encodeUniqueId = encodeURIComponent(linkUniqueId);
			}
			var uri = './document?lawid=' + encodeUniqueId;

			if (spFlag == 1) {
				location.href = uri;
			} else {
				open(uri, linkUniqueId);
			}
		}
	});
	
	//タイムライン表示
	timelineSet();
});